@@ -1,28 +1,36 @@
+
 import numpy as np
 
 from pykin.robots.robot import Robot
 from pykin.utils.error_utils import NotFoundError
+from pykin.utils.transform_utils import get_pose_from_homogeneous
 
 class SingleArm(Robot):
     """
     Initializes a single-armed robot simulation object.
 
     Args:
-        fname (str): path to the urdf file.
+        f_name (str): path to the urdf file.
         offset (Transform): robot init offset
     """
     def __init__(
         self,
-        fname:str,
-        offset=None
+        f_name:str,
+        offset=None,
+        has_gripper=False,
     ):
-        super(SingleArm, self).__init__(fname, offset)
+        super(SingleArm, self).__init__(f_name, offset, has_gripper)
         self._base_name = ""
         self._eef_name  = ""
         self.desired_base_frame = ""
         self._set_joint_limits_upper_and_lower()
         self._init_qpos = np.zeros(self.arm_dof)
         
+        self.info= super()._init_robot_info()
+
+        if has_gripper:
+            self.gripper.info = super()._init_gripper_info()
+        
     def _set_joint_limits_upper_and_lower(self):
         """
         Set joint limits upper and lower
@@ -37,6 +45,36 @@ class SingleArm(Robot):
                 self.joint_limits_lower.append(limit_lower)
                 self.joint_limits_upper.append(limit_upper)
 
+    def get_result_qpos(self, init_qpos, eef_pose, method="LM", max_iter=100):
+        is_limit_qpos = False
+        result_qpos = self.inverse_kin(init_qpos, eef_pose, method=method, max_iter=max_iter)
+        is_limit_qpos = self.check_limit_joint(result_qpos)
+        limit_cnt = 0
+
+        if is_limit_qpos:
+            return result_qpos
+
+        while not is_limit_qpos:
+            limit_cnt += 1
+            if limit_cnt > 3:
+                break
+            result_qpos = self.inverse_kin(np.random.randn(len(init_qpos)), eef_pose, method=method, max_iter=max_iter)
+            is_limit_qpos = self.check_limit_joint(result_qpos)
+        return result_qpos
+
+    def get_info(self, geom="all"):
+        if geom == "all":
+            return self.info
+
+        if geom == "collision":
+            return self.info["collision"]
+        
+        if geom == "visual":
+            return self.info["visual"]
+
+    def get_gripper_init_pose(self):
+        return self.init_fk["right_gripper"].h_mat
+
     def check_limit_joint(self, q_in):
         """
         check q_in within joint limits
@@ -99,7 +137,7 @@ class SingleArm(Robot):
         self.desired_frames = super().generate_desired_frame_recursive(self.desired_base_frame, self.eef_name)
         self._revolute_joint_names = sorted(self.get_revolute_joint_names(self.desired_frames))
 
-    def inverse_kin(self, current_joints, target_pose, method="LM", max_iter=1000):
+    def inverse_kin(self, current_joints, target_pose, method="LM", max_iter=100):
         """
         Returns joint angles obtained by computing IK
         
@@ -112,6 +150,11 @@ class SingleArm(Robot):
         Returns:
             joints (np.array): target joint angles
         """
+        target_pose = np.asarray(target_pose)
+        
+        if target_pose.shape == (4,4):
+            target_pose = get_pose_from_homogeneous(target_pose)
+
         joints = self.kin.inverse_kinematics(
             self.desired_frames,
             current_joints,
@@ -120,7 +163,7 @@ class SingleArm(Robot):
             max_iter)
         return joints
 
-    def get_eef_pose(self, fk=None):
+    def compute_eef_pose(self, fk=None):
         """
         Get end effector's pose
 
@@ -135,7 +178,7 @@ class SingleArm(Robot):
 
         return np.concatenate((fk[self.eef_name].pos, fk[self.eef_name].rot))
 
-    def get_eef_h_mat(self, fk=None):
+    def compute_eef_h_mat(self, fk=None):
         """
         Get end effector's homogeneous marix
 
@@ -150,36 +193,6 @@ class SingleArm(Robot):
 
         return fk[self.eef_name].h_mat
 
-    def get_eef_pos(self, fk=None):
-        """
-        Get end effector's position
-
-        Args:
-            fk(OrderedDict)
-        
-        Returns:
-            vals(dict)
-        """
-        if fk is None:
-            fk = self.init_fk
-
-        return fk[self.eef_name].pos
-
-    def get_eef_ori(self, fk=None):
-        """
-        Get end effector's orientation
-
-        Args:
-            fk(OrderedDict)
-        
-        Returns:
-            vals(dict)
-        """
-        if fk is None:
-            fk = self.init_fk
-
-        return fk[self.eef_name].rot
-
     @property
     def base_name(self):
         return self._base_name
